@@ -21,11 +21,10 @@
 #include "ListRenderer.hxx"
 #include "ListText.hxx"
 #include "config.h"
-#include "options.hxx"
+#include "Options.hxx"
 #include "charset.hxx"
-#include "match.hxx"
-#include "command.hxx"
-#include "colors.hxx"
+#include "Match.hxx"
+#include "Command.hxx"
 #include "paint.hxx"
 #include "screen_status.hxx"
 #include "screen_utils.hxx"
@@ -337,6 +336,10 @@ ListWindow::Find(const ListText &text,
 
 	assert(str != nullptr);
 
+	MatchExpression m;
+	if (!m.Compile(str, false))
+		return false;
+
 	do {
 		while (i < length) {
 			char buffer[1024];
@@ -345,7 +348,7 @@ ListWindow::Find(const ListText &text,
 						     i);
 			assert(label != nullptr);
 
-			if (match_line(label, str)) {
+			if (m(label)) {
 				MoveCursor(i);
 				return true;
 			}
@@ -379,6 +382,10 @@ ListWindow::ReverseFind(const ListText &text,
 	if (length == 0)
 		return false;
 
+	MatchExpression m;
+	if (!m.Compile(str, false))
+		return false;
+
 	do {
 		while (i >= 0) {
 			char buffer[1024];
@@ -387,7 +394,7 @@ ListWindow::ReverseFind(const ListText &text,
 						     i);
 			assert(label != nullptr);
 
-			if (match_line(label, str)) {
+			if (m(label)) {
 				MoveCursor(i);
 				return true;
 			}
@@ -406,34 +413,13 @@ ListWindow::ReverseFind(const ListText &text,
 	return false;
 }
 
-#ifdef NCMPC_MINI
 bool
 ListWindow::Jump(const ListText &text, const char *str)
 {
 	assert(str != nullptr);
 
-	for (unsigned i = 0; i < length; i++) {
-		char buffer[1024];
-		const char *label =
-			text.GetListItemText(buffer, sizeof(buffer),
-					     i);
-		assert(label != nullptr);
-
-		if (g_ascii_strncasecmp(label, str, strlen(str)) == 0) {
-			MoveCursor(i);
-			return true;
-		}
-	}
-	return false;
-}
-#else
-bool
-ListWindow::Jump(const ListText &text, const char *str)
-{
-	assert(str != nullptr);
-
-	GRegex *regex = compile_regex(str, options.jump_prefix_only);
-	if (regex == nullptr)
+	MatchExpression m;
+	if (!m.Compile(str, options.jump_prefix_only))
 		return false;
 
 	for (unsigned i = 0; i < length; i++) {
@@ -443,50 +429,48 @@ ListWindow::Jump(const ListText &text, const char *str)
 					     i);
 		assert(label != nullptr);
 
-		if (match_regex(regex, label)) {
-			g_regex_unref(regex);
+		if (m(label)) {
 			MoveCursor(i);
 			return true;
 		}
 	}
-	g_regex_unref(regex);
+
 	return false;
 }
-#endif
 
 /* perform basic list window commands (movement) */
 bool
-ListWindow::HandleCommand(command_t cmd)
+ListWindow::HandleCommand(Command cmd)
 {
 	switch (cmd) {
-	case CMD_LIST_PREVIOUS:
+	case Command::LIST_PREVIOUS:
 		MoveCursorPrevious();
 		break;
-	case CMD_LIST_NEXT:
+	case Command::LIST_NEXT:
 		MoveCursorNext();
 		break;
-	case CMD_LIST_TOP:
+	case Command::LIST_TOP:
 		MoveCursorTop();
 		break;
-	case CMD_LIST_MIDDLE:
+	case Command::LIST_MIDDLE:
 		MoveCursorMiddle();
 		break;
-	case CMD_LIST_BOTTOM:
+	case Command::LIST_BOTTOM:
 		MoveCursorBottom();
 		break;
-	case CMD_LIST_FIRST:
+	case Command::LIST_FIRST:
 		MoveCursorFirst();
 		break;
-	case CMD_LIST_LAST:
+	case Command::LIST_LAST:
 		MoveCursorLast();
 		break;
-	case CMD_LIST_NEXT_PAGE:
+	case Command::LIST_NEXT_PAGE:
 		MoveCursorNextPage();
 		break;
-	case CMD_LIST_PREVIOUS_PAGE:
+	case Command::LIST_PREVIOUS_PAGE:
 		MoveCursorPreviousPage();
 		break;
-	case CMD_LIST_RANGE_SELECT:
+	case Command::LIST_RANGE_SELECT:
 		if(range_selection)
 		{
 			screen_status_message(_("Range selection disabled"));
@@ -499,16 +483,16 @@ ListWindow::HandleCommand(command_t cmd)
 			range_selection = true;
 		}
 		break;
-	case CMD_LIST_SCROLL_UP_LINE:
+	case Command::LIST_SCROLL_UP_LINE:
 		ScrollUp(1);
 		break;
-	case CMD_LIST_SCROLL_DOWN_LINE:
+	case Command::LIST_SCROLL_DOWN_LINE:
 		ScrollDown(1);
 		break;
-	case CMD_LIST_SCROLL_UP_HALF:
+	case Command::LIST_SCROLL_UP_HALF:
 		ScrollUp((size.height - 1) / 2);
 		break;
-	case CMD_LIST_SCROLL_DOWN_HALF:
+	case Command::LIST_SCROLL_DOWN_HALF:
 		ScrollDown((size.height - 1) / 2);
 		break;
 	default:
@@ -519,33 +503,33 @@ ListWindow::HandleCommand(command_t cmd)
 }
 
 bool
-ListWindow::HandleScrollCommand(command_t cmd)
+ListWindow::HandleScrollCommand(Command cmd)
 {
 	switch (cmd) {
-	case CMD_LIST_SCROLL_UP_LINE:
-	case CMD_LIST_PREVIOUS:
+	case Command::LIST_SCROLL_UP_LINE:
+	case Command::LIST_PREVIOUS:
 		if (start > 0)
 			start--;
 		break;
 
-	case CMD_LIST_SCROLL_DOWN_LINE:
-	case CMD_LIST_NEXT:
+	case Command::LIST_SCROLL_DOWN_LINE:
+	case Command::LIST_NEXT:
 		if (start + size.height < length)
 			start++;
 		break;
 
-	case CMD_LIST_FIRST:
+	case Command::LIST_FIRST:
 		start = 0;
 		break;
 
-	case CMD_LIST_LAST:
+	case Command::LIST_LAST:
 		if (length > size.height)
 			start = length - size.height;
 		else
 			start = 0;
 		break;
 
-	case CMD_LIST_NEXT_PAGE:
+	case Command::LIST_NEXT_PAGE:
 		start += size.height;
 		if (start + size.height > length) {
 			if (length > size.height)
@@ -555,21 +539,21 @@ ListWindow::HandleScrollCommand(command_t cmd)
 		}
 		break;
 
-	case CMD_LIST_PREVIOUS_PAGE:
+	case Command::LIST_PREVIOUS_PAGE:
 		if (start > size.height)
 			start -= size.height;
 		else
 			start = 0;
 		break;
 
-	case CMD_LIST_SCROLL_UP_HALF:
+	case Command::LIST_SCROLL_UP_HALF:
 		if (start > (size.height - 1) / 2)
 			start -= (size.height - 1) / 2;
 		else
 			start = 0;
 		break;
 
-	case CMD_LIST_SCROLL_DOWN_HALF:
+	case Command::LIST_SCROLL_DOWN_HALF:
 		start += (size.height - 1) / 2;
 		if (start + size.height > length) {
 			if (length > size.height)
