@@ -18,13 +18,13 @@
  */
 
 #include "LyricsPage.hxx"
-#include "screen_interface.hxx"
+#include "PageMeta.hxx"
 #include "screen_status.hxx"
 #include "FileBrowserPage.hxx"
 #include "SongPage.hxx"
 #include "Event.hxx"
 #include "i18n.h"
-#include "options.hxx"
+#include "Options.hxx"
 #include "mpdclient.hxx"
 #include "screen.hxx"
 #include "lyrics.hxx"
@@ -32,6 +32,8 @@
 #include "screen_utils.hxx"
 #include "ncu.hxx"
 
+#include <string>
+
 #include <assert.h>
 #include <errno.h>
 #include <sys/stat.h>
@@ -51,7 +53,13 @@ class LyricsPage final : public TextPage {
 
 	struct mpd_song *song = nullptr;
 
-	char *artist = nullptr, *title = nullptr, *plugin_name = nullptr;
+	/**
+	 * These are pointers into the #mpd_song above, and will
+	 * become invalid as soon as the mpd_song_free() is called.
+	 */
+	const char *artist = nullptr, *title = nullptr;
+
+	std::string plugin_name;
 
 	PluginCycle *loader = nullptr;
 
@@ -105,7 +113,7 @@ public:
 	void OnOpen(struct mpdclient &c) override;
 
 	void Update(struct mpdclient &c, unsigned events) override;
-	bool OnCommand(struct mpdclient &c, command_t cmd) override;
+	bool OnCommand(struct mpdclient &c, Command cmd) override;
 
 	const char *GetTitle(char *, size_t) const override;
 };
@@ -123,18 +131,13 @@ LyricsPage::Cancel()
 		loader_timeout = 0;
 	}
 
-	g_free(plugin_name);
-	plugin_name = nullptr;
-
-	g_free(artist);
-	artist = nullptr;
-
-	g_free(title);
-	title = nullptr;
+	plugin_name.clear();
 
 	if (song != nullptr) {
 		mpd_song_free(song);
 		song = nullptr;
+		artist = nullptr;
+		title = nullptr;
 	}
 }
 
@@ -222,7 +225,7 @@ LyricsPage::PluginCallback(std::string &&result, const bool success,
 {
 	assert(loader != nullptr);
 
-	plugin_name = g_strdup(_plugin_name);
+	plugin_name = _plugin_name;
 
 	/* Display result, which may be lyrics or error messages */
 	Set(result.c_str());
@@ -266,12 +269,9 @@ LyricsPage::Load(const struct mpd_song *_song)
 	Cancel();
 	Clear();
 
-	const char *_artist = mpd_song_get_tag(_song, MPD_TAG_ARTIST, 0);
-	const char *_title = mpd_song_get_tag(_song, MPD_TAG_TITLE, 0);
-
 	song = mpd_song_dup(_song);
-	artist = g_strdup(_artist);
-	title = g_strdup(_title);
+	artist = mpd_song_get_tag(song, MPD_TAG_ARTIST, 0);
+	title = mpd_song_get_tag(song, MPD_TAG_TITLE, 0);
 
 	loader = lyrics_load(artist, title, PluginCallback, this);
 
@@ -292,10 +292,10 @@ LyricsPage::Reload()
 	}
 }
 
-static Page *
+static std::unique_ptr<Page>
 lyrics_screen_init(ScreenManager &_screen, WINDOW *w, Size size)
 {
-	return new LyricsPage(_screen, w, size);
+	return std::make_unique<LyricsPage>(_screen, w, size);
 }
 
 void
@@ -345,9 +345,10 @@ LyricsPage::GetTitle(char *str, size_t size) const
 			     _("Lyrics"),
 			     artist, title);
 
-		if (options.lyrics_show_plugin && plugin_name != nullptr &&
+		if (options.lyrics_show_plugin && !plugin_name.empty() &&
 		    (unsigned int) n < size - 1)
-			snprintf(str + n, size - n, " (%s)", plugin_name);
+			snprintf(str + n, size - n, " (%s)",
+				 plugin_name.c_str());
 
 		return str;
 	} else
@@ -409,59 +410,56 @@ LyricsPage::Edit()
 		else if (WEXITSTATUS(status) == 127)
 			screen_status_message(_("Can't start editor"));
 		else
-			screen_status_printf(_("Editor exited unexpectedly (%d)"),
+			screen_status_printf("%s (%d)",
+					     _("Editor exited unexpectedly"),
 					     WEXITSTATUS(status));
 	} else if (WIFSIGNALED(status)) {
-		screen_status_printf(_("Editor exited unexpectedly (signal %d)"),
+		screen_status_printf("%s (signal %d)",
+				     _("Editor exited unexpectedly"),
 				     WTERMSIG(status));
 	}
 }
 
 bool
-LyricsPage::OnCommand(struct mpdclient &c, command_t cmd)
+LyricsPage::OnCommand(struct mpdclient &c, Command cmd)
 {
 	if (TextPage::OnCommand(c, cmd))
 		return true;
 
 	switch(cmd) {
-	case CMD_INTERRUPT:
+	case Command::INTERRUPT:
 		if (loader != nullptr) {
 			Cancel();
 			Clear();
 		}
 		return true;
-	case CMD_SAVE_PLAYLIST:
+	case Command::SAVE_PLAYLIST:
 		if (loader == nullptr && artist != nullptr &&
 		    title != nullptr && Save())
 			/* lyrics for the song were saved on hard disk */
 			screen_status_message (_("Lyrics saved"));
 		return true;
-	case CMD_DELETE:
+	case Command::DELETE:
 		if (loader == nullptr && artist != nullptr &&
 		    title != nullptr) {
-			switch (Delete()) {
-			case true:
-				screen_status_message (_("Lyrics deleted"));
-				break;
-			case false:
-				screen_status_message (_("No saved lyrics"));
-				break;
-			}
+			screen_status_message(Delete()
+					      ? _("Lyrics deleted")
+					      : _("No saved lyrics"));
 		}
 		return true;
-	case CMD_LYRICS_UPDATE:
+	case Command::LYRICS_UPDATE:
 		if (c.song != nullptr)
 			Load(c.song);
 		return true;
-	case CMD_EDIT:
+	case Command::EDIT:
 		Edit();
 		return true;
-	case CMD_SELECT:
+	case Command::SELECT:
 		Reload();
 		return true;
 
 #ifdef ENABLE_SONG_SCREEN
-	case CMD_SCREEN_SONG:
+	case Command::SCREEN_SONG:
 		if (song != nullptr) {
 			screen_song_switch(screen, c, *song);
 			return true;
@@ -469,11 +467,11 @@ LyricsPage::OnCommand(struct mpdclient &c, command_t cmd)
 
 		break;
 #endif
-	case CMD_SCREEN_SWAP:
+	case Command::SCREEN_SWAP:
 		screen.Swap(c, song);
 		return true;
 
-	case CMD_LOCATE:
+	case Command::LOCATE:
 		if (song != nullptr) {
 			screen_file_goto_song(screen, c, *song);
 			return true;
@@ -488,8 +486,10 @@ LyricsPage::OnCommand(struct mpdclient &c, command_t cmd)
 	return false;
 }
 
-const struct screen_functions screen_lyrics = {
+const PageMeta screen_lyrics = {
 	"lyrics",
+	N_("Lyrics"),
+	Command::SCREEN_LYRICS,
 	lyrics_screen_init,
 };
 
