@@ -22,8 +22,10 @@
 #include "mpdclient.hxx"
 #include "callbacks.hxx"
 #include "charset.hxx"
-#include "options.hxx"
-#include "command.hxx"
+#include "Options.hxx"
+#include "Command.hxx"
+#include "Bindings.hxx"
+#include "GlobalBindings.hxx"
 #include "ncu.hxx"
 #include "screen.hxx"
 #include "screen_status.hxx"
@@ -80,7 +82,7 @@ update_xterm_title()
 
 	char tmp[BUFSIZE];
 	const char *new_title = nullptr;
-	if (!options.xterm_title_format.empty() && mpd->playing && song)
+	if (!options.xterm_title_format.empty() && mpd->playing_or_paused && song)
 		new_title = strfsong(tmp, BUFSIZE,
 				     options.xterm_title_format.c_str(), song) > 0
 			? tmp
@@ -162,9 +164,8 @@ timer_reconnect(gcc_unused gpointer data)
 
 	reconnect_source_id = 0;
 
-	screen_status_printf(_("Connecting to %s...  [Press %s to abort]"),
-			     mpd->GetSettingsName().c_str(),
-			     get_key_names(CMD_QUIT, false));
+	screen_status_printf(_("Connecting to %s"),
+			     mpd->GetSettingsName().c_str());
 	doupdate();
 
 	mpd->Connect();
@@ -180,12 +181,12 @@ mpdclient_connected_callback()
 #ifndef NCMPC_MINI
 	/* quit if mpd is pre 0.14 - song id not supported by mpd */
 	auto *connection = mpd->GetConnection();
-	if (mpd_connection_cmp_server_version(connection, 0, 16, 0) < 0) {
+	if (mpd_connection_cmp_server_version(connection, 0, 19, 0) < 0) {
 		const unsigned *version =
 			mpd_connection_get_server_version(connection);
 		screen_status_printf(_("Error: MPD version %d.%d.%d is too old (%s needed)"),
 				     version[0], version[1], version[2],
-				     "0.16.0");
+				     "0.19.0");
 		mpd->Disconnect();
 		doupdate();
 
@@ -265,16 +266,16 @@ void end_input_event()
 }
 
 bool
-do_input_event(command_t cmd)
+do_input_event(Command cmd)
 {
-	if (cmd == CMD_QUIT) {
+	if (cmd == Command::QUIT) {
 		g_main_loop_quit(main_loop);
 		return false;
 	}
 
 	screen->OnCommand(*mpd, cmd);
 
-	if (cmd == CMD_VOLUME_UP || cmd == CMD_VOLUME_DOWN)
+	if (cmd == Command::VOLUME_UP || cmd == Command::VOLUME_DOWN)
 		/* make sure we don't update the volume yet */
 		disable_update_timer();
 
@@ -301,27 +302,14 @@ timer_check_key_bindings(gcc_unused gpointer data)
 {
 	char buf[256];
 
-	if (check_key_bindings(nullptr, buf, sizeof(buf))) {
+	if (GetGlobalKeyBindings().Check(buf, sizeof(buf))) {
 		/* no error: disable this timer for the rest of this
 		   process */
 		check_key_bindings_source_id = 0;
 		return false;
 	}
 
-#ifdef ENABLE_KEYDEF_SCREEN
-	g_strchomp(buf);
-	g_strlcat(buf, " (", sizeof(buf));
-	/* to translators: a key was bound twice in the key editor,
-	   and this is a hint for the user what to press to correct
-	   that */
-	char comment[64];
-	snprintf(comment, sizeof(comment), _("press %s for the key editor"),
-		 get_key_names(CMD_SCREEN_KEYDEF, false));
-	g_strlcat(buf, comment, sizeof(buf));
-	g_strlcat(buf, ")", sizeof(buf));
-#endif
-
-	screen_status_printf("%s", buf);
+	screen_status_message(buf);
 
 	doupdate();
 	return true;
@@ -365,7 +353,7 @@ main(int argc, const char *argv[])
 	read_configuration();
 
 	/* check key bindings */
-	check_key_bindings(nullptr, nullptr, 0);
+	GetGlobalKeyBindings().Check(nullptr, 0);
 #endif
 
 	/* parse command line options - 2 pass */
