@@ -62,18 +62,19 @@ MpdQueue::Move(unsigned dest, unsigned src)
 	items[dest] = std::move(song);
 }
 
-int
-MpdQueue::Find(const struct mpd_song &song) const
+MpdQueue::size_type
+MpdQueue::FindByReference(const struct mpd_song &song) const
 {
-	for (size_type i = 0; i < size(); ++i)
+	for (size_type i = 0;; ++i) {
+		assert(i < size());
+
 		if (&(*this)[i] == &song)
 			return i;
-
-	return -1;
+	}
 }
 
 int
-MpdQueue::FindId(unsigned id) const
+MpdQueue::FindById(unsigned id) const
 {
 	for (size_type i = 0; i < size(); ++i) {
 		const auto &song = (*this)[i];
@@ -85,7 +86,7 @@ MpdQueue::FindId(unsigned id) const
 }
 
 int
-MpdQueue::FindUri(const char *filename) const
+MpdQueue::FindByUri(const char *filename) const
 {
 	for (size_type i = 0; i < size(); ++i) {
 		const auto &song = (*this)[i];
