@@ -18,19 +18,16 @@
  */
 
 #include "TitleBar.hxx"
-#include "colors.hxx"
-#include "options.hxx"
-#include "command.hxx"
+#include "TabBar.hxx"
+#include "Styles.hxx"
+#include "Options.hxx"
 #include "i18n.h"
-#include "charset.hxx"
+#include "util/StringUTF8.hxx"
 
 #include "config.h"
 
 #include <mpd/client.h>
 
-#include <glib.h>
-
-#include <assert.h>
 #include <string.h>
 
 TitleBar::TitleBar(Point p, unsigned width)
@@ -41,24 +38,10 @@ TitleBar::TitleBar(Point p, unsigned width)
 
 #ifdef ENABLE_COLORS
 	if (options.enable_colors)
-		wbkgd(window.w, COLOR_PAIR(COLOR_TITLE));
+		wbkgd(window.w, COLOR_PAIR(Style::TITLE));
 #endif
 }
 
-#ifndef NCMPC_MINI
-static void
-print_hotkey(WINDOW *w, command_t cmd, const char *label)
-{
-	colors_use(w, COLOR_TITLE_BOLD);
-	waddstr(w, get_key_names(cmd, false));
-	colors_use(w, COLOR_TITLE);
-	waddch(w, ':');
-	waddstr(w, label);
-	waddch(w, ' ');
-	waddch(w, ' ');
-}
-#endif
-
 static inline int
 get_volume(const struct mpd_status *status)
 {
@@ -91,40 +74,25 @@ TitleBar::Update(const struct mpd_status *status)
 }
 
 void
-TitleBar::Paint(const char *title) const
+TitleBar::Paint(const PageMeta &current_page_meta, const char *title) const
 {
 	WINDOW *w = window.w;
 
 	wmove(w, 0, 0);
 	wclrtoeol(w);
 
-	if (title[0]) {
-		colors_use(w, COLOR_TITLE_BOLD);
+#ifndef NCMPC_MINI
+	if (options.welcome_screen_list) {
+		PaintTabBar(w, current_page_meta, title);
+	} else {
+#else
+		(void)current_page_meta;
+#endif
+		SelectStyle(w, Style::TITLE_BOLD);
 		mvwaddstr(w, 0, 0, title);
 #ifndef NCMPC_MINI
-	} else {
-#ifdef ENABLE_HELP_SCREEN
-		print_hotkey(w, CMD_SCREEN_HELP, _("Help"));
-#endif
-		print_hotkey(w, CMD_SCREEN_PLAY, _("Queue"));
-		print_hotkey(w, CMD_SCREEN_FILE, _("Browse"));
-#ifdef ENABLE_ARTIST_SCREEN
-		print_hotkey(w, CMD_SCREEN_ARTIST, _("Artist"));
-#endif
-#ifdef ENABLE_SEARCH_SCREEN
-		print_hotkey(w, CMD_SCREEN_SEARCH, _("Search"));
-#endif
-#ifdef ENABLE_LYRICS_SCREEN
-		print_hotkey(w, CMD_SCREEN_LYRICS, _("Lyrics"));
-#endif
-#ifdef ENABLE_OUTPUTS_SCREEN
-		print_hotkey(w, CMD_SCREEN_OUTPUTS, _("Outputs"));
-#endif
-#ifdef ENABLE_CHAT_SCREEN
-		print_hotkey(w, CMD_SCREEN_CHAT, _("Chat"));
-#endif
-#endif
 	}
+#endif
 
 	char buf[32];
 	const char *volume_string;
@@ -135,18 +103,18 @@ TitleBar::Paint(const char *title) const
 		volume_string = buf;
 	}
 
-	colors_use(w, COLOR_TITLE);
+	SelectStyle(w, Style::TITLE);
 	mvwaddstr(w, 0, window.size.width - utf8_width(volume_string),
 		  volume_string);
 
-	colors_use(w, COLOR_LINE);
+	SelectStyle(w, Style::LINE);
 	mvwhline(w, 1, 0, ACS_HLINE, window.size.width);
 	if (flags[0]) {
 		wmove(w, 1, window.size.width - strlen(flags) - 3);
 		waddch(w, '[');
-		colors_use(w, COLOR_LINE_FLAGS);
+		SelectStyle(w, Style::LINE_FLAGS);
 		waddstr(w, flags);
-		colors_use(w, COLOR_LINE);
+		SelectStyle(w, Style::LINE);
 		waddch(w, ']');
 	}
 
