@@ -19,17 +19,22 @@
 
 #include "conf.hxx"
 #include "config.h"
+#include "Bindings.hxx"
+#include "GlobalBindings.hxx"
 #include "defaults.hxx"
 #include "i18n.h"
-#include "command.hxx"
-#include "colors.hxx"
+#include "Command.hxx"
+#include "Styles.hxx"
+#include "BasicColors.hxx"
+#include "CustomColors.hxx"
 #include "screen_list.hxx"
-#include "options.hxx"
+#include "Options.hxx"
+#include "io/Path.hxx"
 #include "util/CharUtil.hxx"
+#include "util/ScopeExit.hxx"
 #include "util/StringStrip.hxx"
 
 #include <assert.h>
-#include <ctype.h>
 #include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -86,6 +91,14 @@
 #define CONF_CHAT_PREFIX "chat-prefix"
 #define CONF_SECOND_COLUMN "second-column"
 
+#ifdef _WIN32
+#define CONFIG_FILENAME "ncmpc.conf"
+#define KEYS_FILENAME "keys.conf"
+#else
+#define CONFIG_FILENAME "config"
+#define KEYS_FILENAME "keys"
+#endif
+
 static bool
 str2bool(char *str)
 {
@@ -158,8 +171,8 @@ parse_key_definition(char *str)
 	while (i < len && str[i] != '=' && !IsWhitespaceFast(str[i]))
 		buf[j++] = str[i++];
 
-	command_t cmd = get_key_command_from_name(buf);
-	if(cmd == CMD_NONE) {
+	const auto cmd = get_key_command_from_name(buf);
+	if(cmd == Command::NONE) {
 		/* the hotkey configuration contains an unknown
 		   command */
 		print_error(_("Unknown command"), buf);
@@ -184,8 +197,7 @@ parse_key_definition(char *str)
 	int key = 0;
 	char *p = buf;
 
-	int keys[MAX_COMMAND_KEYS];
-	memset(keys, 0, sizeof(int)*MAX_COMMAND_KEYS);
+	std::array<int, MAX_COMMAND_KEYS> keys{0};
 	while (i < MAX_COMMAND_KEYS && *p != 0 &&
 	       (key = parse_key_value(p, &p)) >= 0) {
 		keys[i++] = key;
@@ -196,7 +208,8 @@ parse_key_definition(char *str)
 	if (key < 0)
 		return false;
 
-	return assign_keys(cmd, keys);
+	GetGlobalKeyBindings().SetKey(cmd, keys);
+	return true;
 }
 
 static bool
@@ -241,7 +254,7 @@ parse_color(char *str)
 	if (value == nullptr)
 		return false;
 
-	return colors_assign(str, value);
+	return ModifyStyle(str, value);
 }
 
 /**
@@ -272,7 +285,7 @@ parse_color_definition(char *str)
 		return false;
 
 	/* get the command name */
-	short color = colors_str2color(str);
+	short color = ParseColorNameOrNumber(str);
 	if (color < 0) {
 		char buf[MAX_LINE_LENGTH];
 		print_error(_("Bad color name"), buf);
@@ -303,7 +316,8 @@ parse_color_definition(char *str)
 		return false;
 	}
 
-	return colors_define(str, rgb[0], rgb[1], rgb[2]);
+	colors_define(color, rgb[0], rgb[1], rgb[2]);
+	return true;
 }
 #endif
 
@@ -324,11 +338,14 @@ static std::vector<std::string>
 check_screen_list(char *value)
 {
 	char **tmp = g_strsplit_set(value, " \t,", 100);
+	AtScopeExit(tmp) { g_strfreev(tmp); };
+
 	std::vector<std::string> screen;
 	int i = 0;
 
 	while( tmp && tmp[i] ) {
 		char *name = g_ascii_strdown(tmp[i], -1);
+		AtScopeExit(name) { g_free(name); };
 		if (*name != '\0') {
 			if (screen_lookup_name(name) == nullptr) {
 				/* an unknown screen name was specified in the
@@ -338,10 +355,9 @@ check_screen_list(char *value)
 				screen.emplace_back(name);
 			}
 		}
-		g_free(name);
 		i++;
 	}
-	g_strfreev(tmp);
+
 	if (screen.empty())
 		return DEFAULT_SCREEN_LIST;
 
@@ -570,7 +586,7 @@ parse_line(char *line)
 }
 
 static int
-read_rc_file(char *filename)
+read_rc_file(const char *filename)
 {
 	assert(filename != nullptr);
 
@@ -592,153 +608,164 @@ read_rc_file(char *filename)
 	return 0;
 }
 
-bool
-check_user_conf_dir()
+/**
+ * Find or create the directory for writing configuration files.
+ *
+ * @return the absolute path; an empty string indicates that no
+ * directory could be created
+ */
+static std::string
+MakeUserConfigPath(const char *filename)
 {
-	char *directory = g_build_filename(g_get_home_dir(), "." PACKAGE, nullptr);
+	const auto directory = BuildPath(g_get_user_config_dir(), PACKAGE);
 
-	if (g_file_test(directory, G_FILE_TEST_IS_DIR)) {
-		g_free(directory);
-		return true;
-	}
-
-	bool success = g_mkdir(directory, 0755) == 0;
-	g_free(directory);
-	return success;
+	return g_file_test(directory.c_str(), G_FILE_TEST_IS_DIR) ||
+		g_mkdir(directory.c_str(), 0755) == 0
+		? BuildPath(directory, filename)
+		: std::string();
 }
 
-char *
-build_user_conf_filename()
+std::string
+MakeKeysPath()
 {
-#ifdef WIN32
-	return g_build_filename(g_get_user_config_dir(), PACKAGE, "ncmpc.conf", nullptr);
-#else
-	return g_build_filename(g_get_home_dir(), "." PACKAGE, "config", nullptr);
+	return MakeUserConfigPath(KEYS_FILENAME);
+}
+
+#ifndef _WIN32
+
+std::string
+GetHomeConfigPath()
+{
+	return BuildPath(g_get_home_dir(), "." PACKAGE, CONFIG_FILENAME);
+}
+
 #endif
+
+std::string
+GetUserConfigPath()
+{
+	return BuildPath(g_get_user_config_dir(), PACKAGE, CONFIG_FILENAME);
 }
 
-char *
-build_system_conf_filename()
+std::string
+GetSystemConfigPath()
 {
-#ifdef WIN32
+#ifdef _WIN32
 	const gchar* const *system_data_dirs;
-	gchar *pathname = nullptr;
 
 	for (system_data_dirs = g_get_system_config_dirs (); *system_data_dirs != nullptr; system_data_dirs++)
 	{
-		pathname = g_build_filename(*system_data_dirs, PACKAGE, "ncmpc.conf", nullptr);
-		if (g_file_test(pathname, G_FILE_TEST_EXISTS))
-		{
-			break;
-		}
-		else
-		{
-			g_free (pathname);
-			pathname = nullptr;
-		}
+		auto path = BuildPath(*system_data_dirs, PACKAGE, CONFIG_FILENAME);
+		if (g_file_test(path.c_str(), G_FILE_TEST_EXISTS))
+			return path;
 	}
-	return pathname;
+	return {};
 #else
-	return g_build_filename(SYSCONFDIR, PACKAGE, "config", nullptr);
+	return BuildPath(SYSCONFDIR, PACKAGE, CONFIG_FILENAME);
 #endif
 }
 
-char *
-build_user_key_binding_filename()
+#ifndef _WIN32
+
+gcc_pure
+static std::string
+GetHomeKeysPath()
 {
-#ifdef WIN32
-	return g_build_filename(g_get_user_config_dir(), PACKAGE, "keys.conf", nullptr);
-#else
-	return g_build_filename(g_get_home_dir(), "." PACKAGE, "keys", nullptr);
-#endif
+	return BuildPath(g_get_home_dir(), "." PACKAGE, KEYS_FILENAME);
 }
 
-static char *
-g_build_system_key_binding_filename()
+#endif
+
+gcc_pure
+static std::string
+GetUserKeysPath()
 {
-#ifdef WIN32
+	return BuildPath(g_get_user_config_dir(), PACKAGE, KEYS_FILENAME);
+}
+
+gcc_pure
+static std::string
+GetSystemKeysPath()
+{
+#ifdef _WIN32
 	const gchar* const *system_data_dirs;
-	gchar *pathname = nullptr;
 
 	for (system_data_dirs = g_get_system_config_dirs (); *system_data_dirs != nullptr; system_data_dirs++)
 	{
-		pathname = g_build_filename(*system_data_dirs, PACKAGE, "keys.conf", nullptr);
-		if (g_file_test(pathname, G_FILE_TEST_EXISTS))
-		{
-			break;
-		}
-		else
-		{
-			g_free (pathname);
-			pathname = nullptr;
-		}
+		auto path = BuildPath(*system_data_dirs, PACKAGE, KEYS_FILENAME);
+		if (g_file_test(pathname.c_str(), G_FILE_TEST_EXISTS))
+			return path;
 	}
-	return pathname;
+	return {}
 #else
-	return g_build_filename(SYSCONFDIR, PACKAGE, "keys", nullptr);
+	return BuildPath(SYSCONFDIR, PACKAGE, KEYS_FILENAME);
 #endif
 }
 
-static char *
+static std::string
 find_config_file()
 {
 	/* check for command line configuration file */
 	if (!options.config_file.empty())
-		return g_strdup(options.config_file.c_str());
+		return options.config_file;
 
-	/* check for user configuration ~/.ncmpc/config */
-	char *filename = build_user_conf_filename();
-	if (g_file_test(filename, G_FILE_TEST_IS_REGULAR))
+	/* check for user configuration ~/.config/ncmpc/config */
+	auto filename = GetUserConfigPath();
+	if (g_file_test(filename.c_str(), G_FILE_TEST_IS_REGULAR))
 		return filename;
 
-	g_free(filename);
+#ifndef _WIN32
+	/* check for user configuration ~/.ncmpc/config */
+	filename = GetHomeConfigPath();
+	if (g_file_test(filename.c_str(), G_FILE_TEST_IS_REGULAR))
+		return filename;
+#endif
 
 	/* check for  global configuration SYSCONFDIR/ncmpc/config */
-	filename = build_system_conf_filename();
-	if (g_file_test(filename, G_FILE_TEST_IS_REGULAR))
+	filename = GetSystemConfigPath();
+	if (g_file_test(filename.c_str(), G_FILE_TEST_IS_REGULAR))
 		return filename;
 
-	g_free(filename);
-	return nullptr;
+	return {};
 }
 
-static char *
+static std::string
 find_keys_file()
 {
 	/* check for command line key binding file */
 	if (!options.key_file.empty())
-		return g_strdup(options.key_file.c_str());
+		return options.key_file;
 
-	/* check for  user key bindings ~/.ncmpc/keys */
-	char *filename = build_user_key_binding_filename();
-	if (g_file_test(filename, G_FILE_TEST_IS_REGULAR))
+	/* check for user key bindings ~/.config/ncmpc/keys */
+	auto filename = GetUserKeysPath();
+	if (g_file_test(filename.c_str(), G_FILE_TEST_IS_REGULAR))
 		return filename;
 
-	g_free(filename);
+#ifndef _WIN32
+	/* check for  user key bindings ~/.ncmpc/keys */
+	filename = GetHomeKeysPath();
+	if (g_file_test(filename.c_str(), G_FILE_TEST_IS_REGULAR))
+		return filename;
+#endif
 
 	/* check for  global key bindings SYSCONFDIR/ncmpc/keys */
-	filename = g_build_system_key_binding_filename();
-	if (g_file_test(filename, G_FILE_TEST_IS_REGULAR))
+	filename = GetSystemKeysPath();
+	if (g_file_test(filename.c_str(), G_FILE_TEST_IS_REGULAR))
 		return filename;
 
-	g_free(filename);
-	return nullptr;
+	return {};
 }
 
 void
 read_configuration()
 {
 	/* load configuration */
-	char *filename = find_config_file();
-	if (filename != nullptr) {
-		read_rc_file(filename);
-		g_free(filename);
-	}
+	auto filename = find_config_file();
+	if (!filename.empty())
+		read_rc_file(filename.c_str());
 
 	/* load key bindings */
 	filename = find_keys_file();
-	if (filename != nullptr) {
-		read_rc_file(filename);
-		g_free(filename);
-	}
+	if (!filename.empty())
+		read_rc_file(filename.c_str());
 }
