@@ -18,8 +18,8 @@
  */
 
 #include "hscroll.hxx"
+#include "Styles.hxx"
 #include "charset.hxx"
-#include "ncfix.h"
 #include "Event.hxx"
 
 #include <algorithm>
@@ -36,21 +36,20 @@ hscroll::TimerCallback()
 }
 
 void
-hscroll::Set(unsigned _x, unsigned _y, unsigned _width, const char *_text)
+hscroll::Set(unsigned _x, unsigned _y, unsigned _width, const char *_text,
+	     Style _style, attr_t _attr)
 {
 	assert(w != nullptr);
 	assert(_text != nullptr);
 
 	x = _x;
 	y = _y;
+	style = _style;
+	attr = _attr;
 
 	if (!basic.Set(_width, _text))
 		return;
 
-	/* obtain the ncurses attributes and the current color, store
-	   them */
-	fix_wattr_get(w, &attrs, &pair, nullptr);
-
 	if (source_id == 0)
 		source_id = ScheduleTimeout<hscroll, &hscroll::TimerCallback>(std::chrono::seconds(1), *this);
 }
@@ -70,17 +69,15 @@ hscroll::Paint() const
 	assert(w != nullptr);
 	assert(basic.IsDefined());
 
-	/* set stored attributes and color */
-	attr_t old_attrs;
-	short old_pair;
-	fix_wattr_get(w, &old_attrs, &old_pair, nullptr);
-	wattr_set(w, attrs, pair, nullptr);
+	SelectStyle(w, style);
+
+	if (attr != 0)
+		wattron(w, attr);
 
 	/* scroll the string, and draw it */
-	char *p = basic.ScrollString();
-	mvwaddstr(w, y, x, p);
-	g_free(p);
+	const auto s = basic.ScrollString();
+	mvwaddnstr(w, y, x, s.first, s.second);
 
-	/* restore previous attributes and color */
-	wattr_set(w, old_attrs, old_pair, nullptr);
+	if (attr != 0)
+		wattroff(w, attr);
 }
