@@ -18,11 +18,14 @@
  */
 
 #include "plugin.hxx"
-#include "Compiler.h"
+#include "io/Path.hxx"
+#include "util/Compiler.h"
+#include "util/UriUtil.hxx"
 
 #include <glib.h>
 
 #include <algorithm>
+#include <memory>
 
 #include <assert.h>
 #include <stdlib.h>
@@ -64,7 +67,7 @@ struct PluginCycle {
 	PluginList *list;
 
 	/** arguments passed to execv() */
-	char **argv;
+	std::unique_ptr<char *[]> argv;
 
 	/** caller defined callback function */
 	plugin_callback_t callback;
@@ -87,28 +90,21 @@ struct PluginCycle {
 	/** list of all error messages from failed plugins */
 	std::string all_errors;
 
-	PluginCycle(PluginList &_list, char **_argv,
+	PluginCycle(PluginList &_list, std::unique_ptr<char *[]> &&_argv,
 		    plugin_callback_t _callback, void *_callback_data)
-		:list(&_list), argv(_argv),
+		:list(&_list), argv(std::move(_argv)),
 		 callback(_callback), callback_data(_callback_data),
 		 next_plugin(0) {}
-
-	~PluginCycle() {
-		/* free argument list */
-		for (unsigned i = 0; i == 0 || argv[i] != nullptr; ++i)
-			g_free(argv[i]);
-		g_free(argv);
-	}
 };
 
 static bool
-register_plugin(PluginList *list, char *path)
+register_plugin(PluginList *list, std::string &&path)
 {
 	struct stat st;
-	if (stat(path, &st) < 0)
+	if (stat(path.c_str(), &st) < 0)
 		return false;
 
-	list->plugins.emplace_back(path);
+	list->plugins.emplace_back(std::move(path));
 	return true;
 }
 
@@ -121,9 +117,7 @@ plugin_list_load_directory(PluginList *list, const char *path)
 
 	const char *name;
 	while ((name = g_dir_read_name(dir)) != nullptr) {
-		char *plugin = g_build_filename(path, name, nullptr);
-		register_plugin(list, plugin);
-		g_free(plugin);
+		register_plugin(list, BuildPath(path, name));
 	}
 
 	g_dir_close(dir);
@@ -244,8 +238,7 @@ start_plugin(PluginCycle *cycle, const char *plugin_path)
 
 	/* set new program name, but free the one from the previous
 	   plugin */
-	g_free(cycle->argv[0]);
-	cycle->argv[0] = g_path_get_basename(plugin_path);
+	cycle->argv[0] = const_cast<char *>(GetUriFilename(plugin_path));
 
 	int fds_stdout[2];
 	if (pipe(fds_stdout) < 0)
@@ -278,7 +271,7 @@ start_plugin(PluginCycle *cycle, const char *plugin_path)
 		close(0);
 		/* XXX close other fds? */
 
-		execv(plugin_path, cycle->argv);
+		execv(plugin_path, cycle->argv.get());
 		_exit(1);
 	}
 
@@ -319,7 +312,7 @@ next_plugin(PluginCycle *cycle)
 	}
 }
 
-static char **
+static auto
 make_argv(const char*const* args)
 {
 	unsigned num = 0;
@@ -327,18 +320,20 @@ make_argv(const char*const* args)
 		++num;
 	num += 2;
 
-	char **ret = g_new(char *, num);
+	std::unique_ptr<char *[]> result(new char *[num]);
+
+	char **ret = result.get();
 
 	/* reserve space for the program name */
 	*ret++ = nullptr;
 
 	while (*args != nullptr)
-		*ret++ = g_strdup(*args++);
+		*ret++ = const_cast<char *>(*args++);
 
 	/* end of argument vector */
 	*ret++ = nullptr;
 
-	return ret - num;
+	return result;
 }
 
 PluginCycle *
