@@ -21,7 +21,7 @@
 #include "Page.hxx"
 #include "config.h"
 #include "mpdclient.hxx"
-#include "options.hxx"
+#include "Options.hxx"
 #include "player_command.hxx"
 
 #include <mpd/client.h>
@@ -31,14 +31,10 @@
 void
 ScreenManager::PaintTopWindow()
 {
-	const char *title = "";
-#ifndef NCMPC_MINI
-	if (welcome_source_id == 0)
-#endif
-		title = current_page->second->GetTitle(buf, buf_size);
+	const char *title = current_page->second->GetTitle(buf, buf_size);
 	assert(title != nullptr);
 
-	title_bar.Paint(title);
+	title_bar.Paint(GetCurrentPageMeta(), title);
 }
 
 void
