@@ -23,13 +23,15 @@
 #include "screen_utils.hxx"
 #include "Point.hxx"
 #include "config.h"
+#include "util/LocaleString.hxx"
+
+#include <string>
 
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
-#include <glib.h>
 
-#if (defined(HAVE_CURSES_ENHANCED) || defined(ENABLE_MULTIBYTE)) && !defined(WIN32)
+#if (defined(HAVE_CURSES_ENHANCED) || defined(ENABLE_MULTIBYTE)) && !defined(_WIN32)
 #include <sys/poll.h>
 #endif
 
@@ -74,25 +76,42 @@ struct wreadln {
 
 	wreadln(WINDOW *_w, bool _masked)
 		:w(_w), masked(_masked) {}
+
+	/** draw line buffer and update cursor position */
+	void Paint() const;
+
+	/** returns the screen column where the cursor is located */
+	gcc_pure
+	unsigned GetCursorColumn() const;
+
+	/** move the cursor one step to the right */
+	void MoveCursorRight();
+
+	/** move the cursor one step to the left */
+	void MoveCursorLeft();
+
+	/** move the cursor to the end of the line */
+	void MoveCursorToEnd();
+
+	void InsertByte(int key);
+	void DeleteChar(size_t x);
+	void DeleteChar() {
+		DeleteChar(cursor);
+	}
 };
 
 /** max items stored in the history list */
-static const guint wrln_max_history_length = 32;
+static constexpr std::size_t wrln_max_history_length = 32;
 
 /** converts a byte position to a screen column */
+gcc_pure
 static unsigned
 byte_to_screen(const char *data, size_t x)
 {
 #if defined(HAVE_CURSES_ENHANCED) || defined(ENABLE_MULTIBYTE)
 	assert(x <= strlen(data));
 
-	char *dup = g_strndup(data, x);
-	char *p = replace_locale_to_utf8(dup);
-
-	unsigned width = utf8_width(p);
-	g_free(p);
-
-	return width;
+	return StringWidthMB(data, x);
 #else
 	(void)data;
 
@@ -101,25 +120,20 @@ byte_to_screen(const char *data, size_t x)
 }
 
 /** finds the first character which doesn't fit on the screen */
+gcc_pure
 static size_t
 screen_to_bytes(const char *data, unsigned width)
 {
 #if defined(HAVE_CURSES_ENHANCED) || defined(ENABLE_MULTIBYTE)
 	size_t length = strlen(data);
-	char *dup = g_strdup(data);
 
 	while (true) {
-		dup[length] = 0;
-		unsigned p_width = locale_width(dup);
+		unsigned p_width = StringWidthMB(data, length);
 		if (p_width <= width)
-			break;
+			return length;
 
 		--length;
 	}
-
-	g_free(dup);
-
-	return length;
 #else
 	(void)data;
 
@@ -127,16 +141,15 @@ screen_to_bytes(const char *data, unsigned width)
 #endif
 }
 
-/** returns the screen column where the cursor is located */
-static unsigned
-cursor_column(const struct wreadln *wr)
+unsigned
+wreadln::GetCursorColumn() const
 {
-	return byte_to_screen(wr->value.data() + wr->start,
-			      wr->cursor - wr->start);
+	return byte_to_screen(value.data() + start, cursor - start);
 }
 
 /** returns the offset in the string to align it at the right border
     of the screen */
+gcc_pure
 static inline size_t
 right_align_bytes(const char *data, size_t right, unsigned width)
 {
@@ -145,26 +158,13 @@ right_align_bytes(const char *data, size_t right, unsigned width)
 
 	assert(right <= strlen(data));
 
-	char *dup = g_strndup(data, right);
-
-	while (dup[start] != 0) {
-		char *p = locale_to_utf8(dup + start);
-		unsigned p_width = utf8_width(p);
-
-		if (p_width < width) {
-			g_free(p);
+	while (start < right) {
+		if (StringWidthMB(data + start, right - start) < width)
 			break;
-		}
 
-		gunichar c = g_utf8_get_char(p);
-		p[g_unichar_to_utf8(c, nullptr)] = 0;
-
-		start += strlen(Utf8ToLocale(p).c_str());
-		g_free(p);
+		start += CharSizeMB(data + start, right - start);
 	}
 
-	g_free(dup);
-
 	return start;
 #else
 	(void)data;
@@ -173,130 +173,61 @@ right_align_bytes(const char *data, size_t right, unsigned width)
 #endif
 }
 
-/** returns the size (in bytes) of the next character */
-static inline size_t
-next_char_size(const char *data)
+void
+wreadln::MoveCursorRight()
 {
-#if defined(HAVE_CURSES_ENHANCED) || defined(ENABLE_MULTIBYTE)
-	char *p = locale_to_utf8(data);
-
-	gunichar c = g_utf8_get_char(p);
-	p[g_unichar_to_utf8(c, nullptr)] = 0;
-	size_t size = strlen(Utf8ToLocale(p).c_str());
-	g_free(p);
-
-	return size;
-#else
-	(void)data;
-
-	return 1;
-#endif
-}
-
-/** returns the size (in bytes) of the previous character */
-static inline size_t
-prev_char_size(const char *data, size_t x)
-{
-#if defined(HAVE_CURSES_ENHANCED) || defined(ENABLE_MULTIBYTE)
-	assert(x > 0);
-
-	char *p = locale_to_utf8(data);
-
-	char *q = p;
-	while (true) {
-		gunichar c = g_utf8_get_char(q);
-		size_t size = g_unichar_to_utf8(c, nullptr);
-		if (size > x)
-			size = x;
-		x -= size;
-		if (x == 0) {
-			g_free(p);
-			return size;
-		}
-
-		q += size;
-	}
-#else
-	(void)data;
-	(void)x;
-
-	return 1;
-#endif
-}
-
-/* move the cursor one step to the right */
-static inline void cursor_move_right(struct wreadln *wr)
-{
-	if (wr->cursor == wr->value.length())
+	if (cursor == value.length())
 		return;
 
-	size_t size = next_char_size(wr->value.data() + wr->cursor);
-	wr->cursor += size;
-	if (cursor_column(wr) >= wr->width)
-		wr->start = right_align_bytes(wr->value.c_str(),
-					      wr->cursor, wr->width);
+	size_t size = CharSizeMB(value.data() + cursor,
+				 value.length() - cursor);
+	cursor += size;
+	if (GetCursorColumn() >= width)
+		start = right_align_bytes(value.c_str(), cursor, width);
 }
 
-/* move the cursor one step to the left */
-static inline void cursor_move_left(struct wreadln *wr)
+void
+wreadln::MoveCursorLeft()
 {
-	if (wr->cursor == 0)
-		return;
-
-	size_t size = prev_char_size(wr->value.c_str(), wr->cursor);
-	assert(wr->cursor >= size);
-	wr->cursor -= size;
-	if (wr->cursor < wr->start)
-		wr->start = wr->cursor;
+	const char *v = value.c_str();
+	const char *new_cursor = PrevCharMB(v, v + cursor);
+	cursor = new_cursor - v;
+	if (cursor < start)
+		start = cursor;
 }
 
-/* move the cursor to the end of the line */
-static inline void cursor_move_to_eol(struct wreadln *wr)
+void
+wreadln::MoveCursorToEnd()
 {
-	wr->cursor = wr->value.length();
-	if (cursor_column(wr) >= wr->width)
-		wr->start = right_align_bytes(wr->value.c_str(),
-					      wr->cursor, wr->width);
+	cursor = value.length();
+	if (GetCursorColumn() >= width)
+		start = right_align_bytes(value.c_str(),
+					      cursor, width);
 }
 
-/* draw line buffer and update cursor position */
-static inline void drawline(const struct wreadln *wr)
+void
+wreadln::Paint() const
 {
-	wmove(wr->w, wr->point.y, wr->point.x);
+	wmove(w, point.y, point.x);
 	/* clear input area */
-	whline(wr->w, ' ', wr->width);
+	whline(w, ' ', width);
 	/* print visible part of the line buffer */
-	if (wr->masked)
-		whline(wr->w, '*', utf8_width(wr->value.c_str() + wr->start));
+	if (masked)
+		whline(w, '*', StringWidthMB(value.c_str() + start));
 	else
-		waddnstr(wr->w, wr->value.c_str() + wr->start,
-			 screen_to_bytes(wr->value.c_str(), wr->width));
+		waddnstr(w, value.c_str() + start,
+			 screen_to_bytes(value.c_str(), width));
 	/* move the cursor to the correct position */
-	wmove(wr->w, wr->point.y, wr->point.x + cursor_column(wr));
+	wmove(w, point.y, point.x + GetCursorColumn());
 	/* tell ncurses to redraw the screen */
 	doupdate();
 }
 
-#if (defined(HAVE_CURSES_ENHANCED) || defined(ENABLE_MULTIBYTE)) && !defined(WIN32)
-static bool
-multibyte_is_complete(const char *p, size_t length)
-{
-	char *q = g_locale_to_utf8(p, length,
-				   nullptr, nullptr, nullptr);
-	if (q != nullptr) {
-		g_free(q);
-		return true;
-	} else {
-		return false;
-	}
-}
-#endif
-
-static void
-wreadln_insert_byte(struct wreadln *wr, gint key)
+void
+wreadln::InsertByte(int key)
 {
 	size_t length = 1;
-#if (defined(HAVE_CURSES_ENHANCED) || defined(ENABLE_MULTIBYTE)) && !defined (WIN32)
+#if (defined(HAVE_CURSES_ENHANCED) || defined(ENABLE_MULTIBYTE)) && !defined(_WIN32)
 	char buffer[32] = { (char)key };
 	struct pollfd pfd = {
 		.fd = 0,
@@ -307,7 +238,7 @@ wreadln_insert_byte(struct wreadln *wr, gint key)
 	/* wide version: try to complete the multibyte sequence */
 
 	while (length < sizeof(buffer)) {
-		if (multibyte_is_complete(buffer, length))
+		if (!IsIncompleteCharMB(buffer, length))
 			/* sequence is complete */
 			break;
 
@@ -317,35 +248,33 @@ wreadln_insert_byte(struct wreadln *wr, gint key)
 			/* no more input from keyboard */
 			break;
 
-		buffer[length++] = wgetch(wr->w);
+		buffer[length++] = wgetch(w);
 	}
 
-	wr->value.insert(wr->cursor, buffer, length);
+	value.insert(cursor, buffer, length);
 
 #else
-	wr->value.insert(wr->cursor, key);
+	value.insert(cursor, key);
 #endif
 
-	wr->cursor += length;
-	if (cursor_column(wr) >= wr->width)
-		wr->start = right_align_bytes(wr->value.c_str(),
-					      wr->cursor, wr->width);
+	cursor += length;
+	if (GetCursorColumn() >= width)
+		start = right_align_bytes(value.c_str(), cursor, width);
 }
 
-static void
-wreadln_delete_char(struct wreadln *wr, size_t x)
+void
+wreadln::DeleteChar(size_t x)
 {
-	assert(x < wr->value.length());
+	assert(x < value.length());
 
-	size_t length = next_char_size(&wr->value[x]);
-	wr->value.erase(x, length);
+	size_t length = CharSizeMB(value.data() + x, value.length() - x);
+	value.erase(x, length);
 }
 
 /* libcurses version */
 
 static std::string
 _wreadln(WINDOW *w,
-	 const char *prompt,
 	 const char *initial_value,
 	 unsigned x1,
 	 History *history,
@@ -359,15 +288,8 @@ _wreadln(WINDOW *w,
 	(void)completion;
 #endif
 
-	/* turn off echo */
-	noecho();
 	/* make sure the cursor is visible */
 	curs_set(1);
-	/* print prompt string */
-	if (prompt) {
-		waddstr(w, prompt);
-		waddstr(w, ": ");
-	}
 	/* retrieve y and x0 position */
 	getyx(w, wr.point.y, wr.point.x);
 	/* check the x1 value */
@@ -391,16 +313,16 @@ _wreadln(WINDOW *w,
 			--hlist;
 			wr.value = *hlist;
 		}
-		cursor_move_to_eol(&wr);
-		drawline(&wr);
+		wr.MoveCursorToEnd();
+		wr.Paint();
 	} else if (initial_value) {
 		/* copy the initial value to the line buffer */
 		wr.value = initial_value;
-		cursor_move_to_eol(&wr);
-		drawline(&wr);
+		wr.MoveCursorToEnd();
+		wr.Paint();
 	}
 
-	gint key = 0;
+	int key = 0;
 	while (key != 13 && key != '\n') {
 		key = wgetch(w);
 
@@ -425,7 +347,7 @@ _wreadln(WINDOW *w,
 				auto r = completion->Complete(wr.value.c_str());
 				if (!r.new_prefix.empty()) {
 					wr.value = std::move(r.new_prefix);
-					cursor_move_to_eol(&wr);
+					wr.MoveCursorToEnd();
 				} else
 					screen_bell();
 
@@ -443,11 +365,11 @@ _wreadln(WINDOW *w,
 
 		case KEY_LEFT:
 		case KEY_CTRL_B:
-			cursor_move_left(&wr);
+			wr.MoveCursorLeft();
 			break;
 		case KEY_RIGHT:
 		case KEY_CTRL_F:
-			cursor_move_right(&wr);
+			wr.MoveCursorRight();
 			break;
 		case KEY_HOME:
 		case KEY_CTRL_A:
@@ -456,7 +378,7 @@ _wreadln(WINDOW *w,
 			break;
 		case KEY_END:
 		case KEY_CTRL_E:
-			cursor_move_to_eol(&wr);
+			wr.MoveCursorToEnd();
 			break;
 		case KEY_CTRL_K:
 			wr.value.erase(wr.cursor);
@@ -469,28 +391,28 @@ _wreadln(WINDOW *w,
 			/* Firstly remove trailing spaces. */
 			for (; wr.cursor > 0 && wr.value[wr.cursor - 1] == ' ';)
 			{
-				cursor_move_left(&wr);
-				wreadln_delete_char(&wr, wr.cursor);
+				wr.MoveCursorLeft();
+				wr.DeleteChar();
 			}
 			/* Then remove word until next space. */
 			for (; wr.cursor > 0 && wr.value[wr.cursor - 1] != ' ';)
 			{
-				cursor_move_left(&wr);
-				wreadln_delete_char(&wr, wr.cursor);
+				wr.MoveCursorLeft();
+				wr.DeleteChar();
 			}
 			break;
 		case 127:
 		case KEY_BCKSPC:	/* handle backspace: copy all */
 		case KEY_BACKSPACE:	/* chars starting from curpos */
 			if (wr.cursor > 0) { /* - 1 from buf[n+1] to buf   */
-				cursor_move_left(&wr);
-				wreadln_delete_char(&wr, wr.cursor);
+				wr.MoveCursorLeft();
+				wr.DeleteChar();
 			}
 			break;
 		case KEY_DC:		/* handle delete key. As above */
 		case KEY_CTRL_D:
 			if (wr.cursor < wr.value.length())
-				wreadln_delete_char(&wr, wr.cursor);
+				wr.DeleteChar();
 			break;
 		case KEY_UP:
 		case KEY_CTRL_P:
@@ -504,7 +426,7 @@ _wreadln(WINDOW *w,
 				--hlist;
 				wr.value = *hlist;
 			}
-			cursor_move_to_eol(&wr);
+			wr.MoveCursorToEnd();
 			break;
 		case KEY_DOWN:
 		case KEY_CTRL_N:
@@ -514,7 +436,7 @@ _wreadln(WINDOW *w,
 				++hlist;
 				wr.value = *hlist;
 			}
-			cursor_move_to_eol(&wr);
+			wr.MoveCursorToEnd();
 			break;
 
 		case '\n':
@@ -527,10 +449,10 @@ _wreadln(WINDOW *w,
 			break;
 		default:
 			if (key >= 32)
-				wreadln_insert_byte(&wr, key);
+				wr.InsertByte(key);
 		}
 
-		drawline(&wr);
+		wr.Paint();
 	}
 
 	/* update history */
@@ -555,21 +477,19 @@ _wreadln(WINDOW *w,
 
 std::string
 wreadln(WINDOW *w,
-	const char *prompt,
 	const char *initial_value,
 	unsigned x1,
 	History *history,
 	Completion *completion)
 {
-	return  _wreadln(w, prompt, initial_value, x1,
+	return  _wreadln(w, initial_value, x1,
 			 history, completion, false);
 }
 
 std::string
 wreadln_masked(WINDOW *w,
-	       const char *prompt,
 	       const char *initial_value,
 	       unsigned x1)
 {
-	return  _wreadln(w, prompt, initial_value, x1, nullptr, nullptr, true);
+	return  _wreadln(w, initial_value, x1, nullptr, nullptr, true);
 }
